
#include "hashtable.h"
#include "philspel.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <assert.h>

/* Global variable */
HashTable *dictionary;

/* Declare constants 
   Note: max_string_length is 61 and 
	 not 60 since we need room for '/0' 
*/
enum { false = 0, true = 1, byte = 8, max_string_length = 61 };

int main(int argc, char **argv){
  if(argc != 2){
    fprintf(stderr, "Specify a dictionary\n");
    return 0;
  }
  fprintf(stderr, "Creating hashtable\n");
  dictionary = createHashTable(2255, &stringHash, &stringEquals);
  fprintf(stderr, "Loading dictionary %s\n", argv[1]);
  readDictionary(argv[1]);
  fprintf(stderr, "Dictionary loaded\n");
  fprintf(stderr, "Processing stdin\n");
  processInput();
  return 0;
}

/* Return unique hash value associated with string argument */
unsigned int stringHash(void *s){
	char *string = (char*) s;
	unsigned int hash = 0;
	while (*string != '\0') 
		hash += (hash << byte) | ((unsigned int) *string++);
	return hash; 
}

/* Return true (1) if equal and false (0) otherwise */
int stringEquals(void *s1, void *s2){
	return (strcmp((char*) s1, (char*) s2)) ?
		false : true;	
}

/*
 * this function should read in every word in the dictionary and
 * store it in the dictionary.  You should first open the file specified,
 * then read the words one at a time and insert them into the dictionary.
 * Once the file is read in completely, exit.  You will need to allocate
 * (using malloc()) space for each word.  You can assume that no word
 * is longer than 60 characters.  You can NOT assume that the specified
 * file exists.  If the file does NOT exist, you should print some message
 * to standard error and call exit(0) to cleanly exit the program.
 * The function assert will accomplish this by printing the line number.
 * Since the format is one word at a time, with returns in between, you can
 * safely use fscanf() to read in the strings.
 */
void readDictionary(char *name){
	FILE *fp = fopen(name, "r");
	/* Since string is needed after function call we allocate from 
	   the heap.  See example of stack allocation in processInput */

	char *string = malloc(sizeof(char)*max_string_length);
	assert(fp != NULL); /* if not true then terminate program gracefully */
	while ( fscanf(fp, "%s\n", string) != EOF ) { 
	        fprintf(stderr, "%s\n", string);
		/* Function header states it want (void*) therefore we case */
		insertData(dictionary, (void*)string, (void*)string);
		/* Point string to new data */
		string=malloc(sizeof(char)*max_string_length); 
	}
	fclose(fp);
}

/*
 * This should process standard input and copy it to standard output
 * as specified in the handout.  EG, if a standard dictionary was used
 * and the string "this is a teast of  this-proGram" was given to
 * standard input, the output to standard output (stdout) should be
 * "this is a teast [sic] of  this-proGram".  All words should be checked
 * against the dictionary as they are input, again with all but the first
 * letter converted to lowercase, and finally with all letters converted
 * to lowercase.  Only if all 3 cases are not in the dictionary should it
 * be reported as not being found, by appending " [sic]" after the
 * error.
 *
 * Since we care about preserving whitespace, and pass on all non alphabet
 * characters untouched, and with all non alphabet characters acting as
 * word breaks, scanf() is probably insufficient (since it only considers
 * whitespace as breaking strings), so you will probably have
 * to get characters from standard input one at a time.
 *
 * You can assume that no word is longer than 60 characters, but you
 * may have strings of non-alphabetic characters (eg, numbers, punctuation)
 * which are longer than 60 characters
 */
void processInput(){
	/* Since the three strings are not needed outside of this
	   Function we allocate memory from the stack and not the Heap */
        char STRING[max_string_length],
                String[max_string_length], string[max_string_length];
        char ch;
        int i=0;
        while ( (ch=getchar()) != EOF || (ch==EOF && i!=0)) {
                if (isalpha((int) ch)) {
                        STRING[i++] = ch; /* STRING[i]; i++; */
                } else if(i==0) {
                        putchar(ch); /* Print non letter char */
                } else {
                        STRING[i]='\0';  /* NULL TERMINATE STRING */
        		assert(i<max_string_length); 
                        for( ;i>=0; i--) {
                                String[i]=string[i]=tolower((int) STRING[i]);
                        }
			/* Recopy first char for Case 3 in description */
                        String[0]=STRING[0]; 

			/* Lookup string in Hashtable and print accordingly */
                        if ( findData(dictionary, (void*) string) ||
                             findData(dictionary, (void*) String) ||
                             findData(dictionary, (void*) STRING) ) {
				printf("%s", STRING);	
                        } else { 
				printf("%s [sic]", STRING);	
			}
                        if (ch!=EOF) putchar(ch); /* EOF will print space */
                        i=0;
                }
        }
}
